﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Xml;

namespace Xml_Database
{
    public partial class Form1 : Form
    {
        string StrCon = ConfigurationManager.ConnectionStrings["conexaoSql"].ToString();

        public Form1()
        {
            InitializeComponent();
        }

        private void btnLocalizar_Click(object sender, EventArgs e)
        {
            this.ofdlg1.Multiselect = false;
            this.ofdlg1.Title = "Selecionar arquivo";
            ofdlg1.InitialDirectory = @"C:\dados\xml";
            //filtra para exibir somente arquivos xml
            ofdlg1.Filter = "XML (*.XML;*.xml)|*.XML;*.xml|" + "Todos (*.*)|*.*";
            ofdlg1.CheckFileExists = true;
            ofdlg1.CheckPathExists = true;
            ofdlg1.FilterIndex = 1;
            ofdlg1.RestoreDirectory = true;
            ofdlg1.ReadOnlyChecked = true;
            ofdlg1.ShowReadOnly = true;

            if (ofdlg1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    txtArquivo.Text = ofdlg1.FileName;
                }
                catch (Exception ex)
                {
                    // Não pode carregar o arquivo (problemas de permissão)
                    MessageBox.Show("Não é possível acessar o arquivo : " + ofdlg1.FileName
                                    + ". Você pode não ter permissão para ler o arquivo , ou " +
                                    " ele pode estar corrompido.\n\nErro reportado : " + ex.Message);
                }
            }
        }

        private void btnImportar_Click(object sender, EventArgs e)
        {
            string arquivoXML = txtArquivo.Text;

            if (File.Exists(arquivoXML))
            {
                // Converte o arquivo Xml para um DataTable  
                DataTable dt = CriaDataTableXML(arquivoXML);
                if (dt.Columns.Count == 0)
                    dt.ReadXml(arquivoXML);

                // Cria uma Consulta para criar a tabela no SQL Server
                string consultaSQL = CriaConsultaTabela(dt);
                SqlConnection con = new SqlConnection(StrCon);
                con.Open();

                // Deleta a tabela se ela já existir
                SqlCommand cmd = new SqlCommand("IF OBJECT_ID('dbo." + dt.TableName + "', 'U') IS NOT NULL DROP TABLE dbo." + dt.TableName + ";", con);
                cmd.ExecuteNonQuery();

                // Cria a tabela
                cmd = new SqlCommand(consultaSQL, con);
                int verifica = cmd.ExecuteNonQuery();
                if (verifica != 0)
                {
                    // Copia os dados a partir do DataTable para a tabela SQL
                    using (var bulkCopy = new SqlBulkCopy(con.ConnectionString, SqlBulkCopyOptions.KeepIdentity))
                    {
                        // os nomes da coluna do DataTable usado correspondem aos nomes das colunas da tabela SQL,
                        // assim usei um laço foreach simples. No entanto, se os nomes das colunas
                        // não corresponderem, apenas passe qual nome do DataTable corresponde ao nome da coluna SQL 
                        // em ColumnMappings
                        foreach (DataColumn col in dt.Columns)
                        {
                            bulkCopy.ColumnMappings.Add(col.ColumnName, col.ColumnName);
                        }

                        bulkCopy.BulkCopyTimeout = 600;
                        bulkCopy.DestinationTableName = dt.TableName;
                        bulkCopy.WriteToServer(dt);
                    }

                    MessageBox.Show("tabela Criada com Sucesso...");
                }
                con.Close();
            }
        }

        // Converte o arquivo Xml para um DataTable  
        public DataTable CriaDataTableXML(string arquivoXML)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(arquivoXML);
            DataTable Dt = new DataTable();
            try
            {
                Dt.TableName = GetNomeTabela(arquivoXML);
                XmlNode NodoEstructura = doc.DocumentElement.ChildNodes.Cast<XmlNode>().ToList()[0];
                pgbar1.Maximum = NodoEstructura.ChildNodes.Count;
                pgbar1.Value = 0;
                foreach (XmlNode columna in NodoEstructura.ChildNodes)
                {
                    Dt.Columns.Add(columna.Name, typeof(String));
                    Progresso();
                }

                XmlNode Filas = doc.DocumentElement;
                pgbar1.Maximum = Filas.ChildNodes.Count;
                pgbar1.Value = 0;
                foreach (XmlNode Fila in Filas.ChildNodes)
                {
                    List<string> Valores = Fila.ChildNodes.Cast<XmlNode>().ToList().Select(x => x.InnerText).ToList();
                    Dt.Rows.Add(Valores.ToArray());
                    Progresso();
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return Dt;
        }

        // Obtem o nome da tabela a partir do nome do arquivo XML
        public string GetNomeTabela(string arquivo)
        {
            FileInfo fi = new FileInfo(arquivo);
            string nomeTabela = fi.Name.Replace(fi.Extension, "");

            return nomeTabela;
        }

        // Exibe a ação da  ProgressBar
        public void Progresso()
        {
            if (pgbar1.Value < pgbar1.Maximum)
            {
                pgbar1.Value++;
                int percent = (int)(((double)pgbar1.Value / (double)pgbar1.Maximum) * 100);
                pgbar1.CreateGraphics().DrawString(percent.ToString() +
                    "%", new Font("Arial", (float)8.25, FontStyle.Regular), 
                    Brushes.Black, new PointF(pgbar1.Width / 2 - 10, pgbar1.Height / 2 - 7));

                Application.DoEvents();
            }
        }

        public string CriaConsultaTabela(DataTable table)
        {
            string sqlsc = "CREATE TABLE " + table.TableName + "(";
            pgbar1.Maximum = table.Columns.Count;
            pgbar1.Value = 0;
            for (int i = 0; i < table.Columns.Count; i++)
            {
                sqlsc += "[" + table.Columns[i].ColumnName + "]";
                string columnType = table.Columns[i].DataType.ToString();
                switch (columnType)
                {
                    case "System.Int32":
                        sqlsc += " int ";
                        break;
                    case "System.Int64":
                        sqlsc += " bigint ";
                        break;
                    case "System.Int16":
                        sqlsc += " smallint";
                        break;
                    case "System.Byte":
                        sqlsc += " tinyint";
                        break;
                    case "System.Decimal":
                        sqlsc += " decimal ";
                        break;
                    case "System.DateTime":
                        sqlsc += " datetime ";
                        break;
                    case "System.String":
                    default:
                        sqlsc += string.Format(" nvarchar({0}) ", table.Columns[i].MaxLength == -1 ? "max" : table.Columns[i].MaxLength.ToString());
                        break;
                }
                if (table.Columns[i].AutoIncrement)
                    sqlsc += " IDENTITY(" + table.Columns[i].AutoIncrementSeed.ToString() + "," + table.Columns[i].AutoIncrementStep.ToString() + ") ";
                if (!table.Columns[i].AllowDBNull)
                    sqlsc += " NOT NULL ";
                sqlsc += ",";

                Progresso();
            }
            return sqlsc.Substring(0, sqlsc.Length - 1) + "\n)";
        }
    }
}
